@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,12 +19,12 @@ package MongoDB::DeleteResult;
 # ABSTRACT: MongoDB deletion result object
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo;
 use MongoDB::_Constants;
-use Types::Standard qw(
-    Num
+use MongoDB::_Types qw(
+    Numish
 );
 use namespace::clean;
 
@@ -44,7 +42,7 @@ with $_ for qw(
 has deleted_count => (
     is      => 'ro',
     default => 0,
-    isa => Num,
+    isa => Numish,
 );
 
 1;
@@ -61,7 +59,7 @@ MongoDB::DeleteResult - MongoDB deletion result object
 
 =head1 VERSION
 
-version v1.8.3
+version v2.0.0
 
 =head1 SYNOPSIS
 
