@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,7 +19,7 @@ package MongoDB::Op::_Delete;
 # Encapsulate a delete operation; returns a MongoDB::DeleteResult
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo;
 
@@ -95,9 +93,10 @@ sub execute {
                 ),
                 $op_doc,
                 "MongoDB::DeleteResult",
+                "delete",
             )
         )
-        : $link->does_write_commands
+        : $link->supports_write_commands
         ? (
             $self->_send_write_command(
                 $link,
@@ -120,6 +119,7 @@ sub execute {
                 ),
                 $op_doc,
                 "MongoDB::DeleteResult",
+                "delete",
             )->assert
         )
     );
