@@ -1,5 +1,4 @@
-#
-#  Copyright 2016 MongoDB, Inc.
+#  Copyright 2016 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,13 +19,16 @@ package MongoDB::Op::_DropIndexes;
 # Implements index drops; returns a MongoDB::CommandResult
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo;
 
+use MongoDB::Error;
 use MongoDB::Op::_Command;
-use Types::Standard qw(
-  Str
+use Safe::Isa;
+use MongoDB::_Types qw(
+    Numish
+    Stringish
 );
 
 use namespace::clean;
@@ -35,7 +36,12 @@ use namespace::clean;
 has index_name => (
     is       => 'ro',
     required => 1,
-    isa      => Str,
+    isa      => Stringish,
+);
+
+has max_time_ms => (
+    is => 'ro',
+    isa => Numish,
 );
 
 with $_ for qw(
@@ -52,14 +58,33 @@ sub execute {
         query   => [
             dropIndexes => $self->coll_name,
             index       => $self->index_name,
-            ( $link->accepts_wire_version(5) ? ( @{ $self->write_concern->as_args } ) : () ),
+            ( $link->supports_helper_write_concern ? ( @{ $self->write_concern->as_args } ) : () ),
+            (defined($self->max_time_ms)
+                ? (maxTimeMS => $self->max_time_ms)
+                : ()
+            ),
         ],
-        query_flags => {},
-        bson_codec  => $self->bson_codec,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        monitoring_callback => $self->monitoring_callback,
     );
 
-    my $res = $op->execute($link);
-    $res->assert_no_write_concern_error;
+    my $res;
+    eval {
+        $res = $op->execute($link);
+        $res->assert_no_write_concern_error;
+    };
+    # XXX This logic will be a problem for command monitoring - may need to
+    # move into Op::_Command as an 'error_filter' callback or something.
+    if ( my $err = $@ ) {
+        if ( $err->$_isa("MongoDB::DatabaseError") ) {
+            # 2.6 and 3.0 don't have the code, so we fallback to string
+            # matching on the error message
+            return $err->result
+              if $err->code == INDEX_NOT_FOUND() || $err->message =~ /index not found with name/;
+        }
+        die $err;
+    }
 
     return $res;
 }
