@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2016 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,7 +19,7 @@ package MongoDB::Role::_DatabaseErrorThrower;
 # MongoDB interface for providing the last database error
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo::Role;
 
@@ -29,7 +27,7 @@ use MongoDB::Error;
 
 use namespace::clean;
 
-requires qw/last_errmsg last_code last_wtimeout/;
+requires qw/last_errmsg last_code last_wtimeout last_error_labels/;
 
 my $ANY_DUP_KEY = [ DUPLICATE_KEY, DUPLICATE_KEY_UPDATE, DUPLICATE_KEY_CAPPED ];
 my $ANY_NOT_MASTER = [ NOT_MASTER, NOT_MASTER_NO_SLAVE_OK, NOT_MASTER_OR_SECONDARY ];
@@ -42,6 +40,7 @@ sub _throw_database_error {
 
     my $err  = $self->last_errmsg;
     my $code = $self->last_code;
+    my $error_labels = $self->last_error_labels;
 
     if ( grep { $code == $_ } @$ANY_NOT_MASTER || $err =~ /^(?:not master|node is recovering)/ ) {
         $error_class = "MongoDB::NotMasterError";
@@ -49,6 +48,9 @@ sub _throw_database_error {
     elsif ( grep { $code == $_ } @$ANY_DUP_KEY ) {
         $error_class = "MongoDB::DuplicateKeyError";
     }
+    elsif ( $code == CURSOR_NOT_FOUND ) {
+        $error_class = "MongoDB::CursorNotFoundError";
+    }
     elsif ( $self->last_wtimeout ) {
         $error_class = "MongoDB::WriteConcernError";
     }
@@ -56,6 +58,7 @@ sub _throw_database_error {
     $error_class->throw(
         result => $self,
         code   => $code || UNKNOWN_ERROR,
+        error_labels => $error_labels,
         ( length($err) ? ( message => $err ) : () ),
     );
 
