@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,12 +19,12 @@ package MongoDB::Role::_InsertPreEncoder;
 # MongoDB interface for pre-encoding and validating docs to insert
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo::Role;
 
-use MongoDB::BSON::_EncodedDoc;
-use MongoDB::OID;
+use BSON::Raw;
+use BSON::OID;
 
 use namespace::clean;
 
@@ -35,13 +33,13 @@ requires qw/bson_codec/;
 # takes MongoDB::_Link and ref of type Document; returns
 # blessed BSON encode doc and the original/generated _id
 sub _pre_encode_insert {
-    my ( $self, $link, $doc, $invalid_chars ) = @_;
+    my ( $self, $max_bson_size, $doc, $invalid_chars ) = @_;
 
     my $type = ref($doc);
 
     my $id = (
           $type eq 'HASH' ? $doc->{_id}
-        : $type eq 'ARRAY' ? do {
+        : $type eq 'ARRAY' || $type eq 'BSON::Doc' ? do {
             my $i;
             for ( $i = 0; $i < @$doc; $i++ ) { last if $doc->[$i] eq '_id' }
             $i < $#$doc ? $doc->[ $i + 1 ] : undef;
@@ -51,16 +49,13 @@ sub _pre_encode_insert {
     );
     if ( ! defined $id ) {
         my $creator = $self->bson_codec->can("create_oid");
-        # "create_oid" is a new codec API method.  If a codec doesn't
-        # have it, we fall back to MongoDB::OID, but use _new_oid for
-        # efficiency as it bypasses Moo construction overhead.
-        $id = $creator ? $creator->() : MongoDB::OID->_new_oid();
+        $id = $creator ? $creator->() : BSON::OID->new();
     }
     my $bson_doc = $self->bson_codec->encode_one(
         $doc,
         {
             invalid_chars => $invalid_chars,
-            max_length    => $link->max_bson_object_size,
+            max_length    => $max_bson_size,
             first_key     => '_id',
             first_value   => $id,
         }
@@ -68,7 +63,7 @@ sub _pre_encode_insert {
 
     # manually bless for speed
     return bless { bson => $bson_doc, metadata => { _id => $id } },
-      "MongoDB::BSON::_EncodedDoc";
+      "BSON::Raw";
 }
 
 1;
