@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,12 +19,14 @@ package MongoDB::UpdateResult;
 # ABSTRACT: MongoDB update result object
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo;
 use MongoDB::_Constants;
+use MongoDB::_Types qw(
+    Numish
+);
 use Types::Standard qw(
-    Num
     Undef
 );
 use namespace::clean;
@@ -45,7 +45,7 @@ with $_ for qw(
 has matched_count => (
     is       => 'ro',
     required => 1,
-    isa      => Num,
+    isa      => Numish,
 );
 
 #pod =attr modified_count
@@ -62,7 +62,7 @@ has matched_count => (
 has modified_count => (
     is       => 'ro',
     required => 1,
-    isa      => (Num|Undef),
+    isa      => (Numish|Undef),
 );
 
 sub has_modified_count {
@@ -95,7 +95,7 @@ MongoDB::UpdateResult - MongoDB update result object
 
 =head1 VERSION
 
-version v1.8.3
+version v2.0.0
 
 =head1 SYNOPSIS
 
