@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use v5.8.0;
 use strict;
@@ -21,11 +19,13 @@ use warnings;
 package MongoDB::_Protocol;
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use MongoDB::_Constants;
 use MongoDB::Error;
 
+use Compress::Zlib ();
+
 use constant {
     OP_REPLY        => 1,    # Reply to a client request. responseTo is set
     OP_MSG          => 1000, # generic msg command followed by a string
@@ -36,6 +36,7 @@ use constant {
     OP_GET_MORE     => 2005, # Get more data from a query. See Cursors
     OP_DELETE       => 2006, # Delete documents
     OP_KILL_CURSORS => 2007, # Tell database client is done with a cursor
+    OP_COMPRESSED   => 2012, # wire compression
 };
 
 use constant {
@@ -62,6 +63,7 @@ use constant {
     P_DELETE       => PERL58 ? "l5Z*l"   : "l<5Z*l<",
     P_KILL_CURSORS => PERL58 ? "l6(a8)*" : "l<6(a8)*",
     P_REPLY_HEADER => PERL58 ? "l5a8l2"  : "l<5a8l<2",
+    P_COMPRESSED   => PERL58 ? "l6C"     : "l<6C",
 };
 
 # struct MsgHeader {
@@ -77,6 +79,108 @@ use constant {
 #     my $msg = pack( P_INSERT, 0, int(rand(2**32-1)), 0, OP_INSERT, 0, $ns ) . $bson_docs;
 #     substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
 
+use constant {
+    # length for MsgHeader
+    P_HEADER_LENGTH =>
+        length(pack P_HEADER, 0, 0, 0, 0),
+    # length for OP_COMPRESSED
+    P_COMPRESSED_PREFIX_LENGTH =>
+        length(pack P_COMPRESSED, 0, 0, 0, 0, 0, 0, 0),
+};
+
+# struct OP_COMPRESSED {
+#     MsgHeader header;             // standard message header
+#     int32_t   originalOpcode;     // wrapped op code
+#     int32_t   uncompressedSize;   // size of deflated wo. header
+#     uint8_t   compressorId;       // compressor
+#     char*     compressedMessage;  // compressed contents
+# };
+
+# decompressors indexed by ID.
+my @DECOMPRESSOR = (
+    # none
+    sub { shift },
+    # snappy (unsupported)
+    undef,
+    # zlib
+    sub { Compress::Zlib::uncompress(shift) },
+);
+
+# construct compressor by name with options
+sub get_compressor {
+    my ($name, $comp_opt) = @_;
+
+    if ($name eq 'none') {
+        return {
+            id => 0,
+            callback => sub { shift },
+        };
+    }
+    elsif ($name eq 'zlib') {
+        my $level = $comp_opt->{zlib_compression_level};
+        $level = undef
+            if defined $level and $level < 0;
+        return {
+            id => 2,
+            callback => sub {
+                return Compress::Zlib::compress(
+                    $_[0],
+                    defined($level) ? $level : (),
+                );
+            },
+        };
+    }
+    else {
+        MongoDB::ProtocolError->throw("Unknown compressor '$name'");
+    }
+}
+
+# compress message
+sub compress {
+    my ($msg, $compressor) = @_;
+
+    my ($len, $request_id, $response_to, $op_code)
+        = unpack(P_HEADER, $msg);
+
+    $msg = substr $msg, P_HEADER_LENGTH;
+
+    my $msg_comp = pack(
+        P_COMPRESSED,
+        0, $request_id, $response_to, OP_COMPRESSED,
+        $op_code,
+        length($msg),
+        $compressor->{id},
+    ).$compressor->{callback}->($msg);
+
+    substr($msg_comp, 0, 4, pack(P_INT32, length($msg_comp)));
+    return $msg_comp;
+}
+
+# attempt to uncompress message
+# messages that aren't OP_COMPRESSED are returned as-is
+sub try_uncompress {
+    my ($msg) = @_;
+
+    my ($len, $request_id, $response_to, $op_code, $orig_op_code, $orig_len, $comp_id)
+        = unpack(P_COMPRESSED, $msg);
+
+    return $msg
+        if $op_code != OP_COMPRESSED;
+
+    $msg = substr $msg, P_COMPRESSED_PREFIX_LENGTH;
+
+    my $decompressor = $DECOMPRESSOR[$comp_id]
+        or MongoDB::ProtocolError->throw("Unknown compressor ID '$comp_id'");
+
+    my $decomp_msg = $decompressor->($msg);
+    my $done =
+        pack(P_HEADER, $orig_len, $request_id, $response_to, $orig_op_code)
+        .$decomp_msg;
+
+    return $done;
+
+}
+
 # struct OP_UPDATE {
 #     MsgHeader header;             // standard message header
 #     int32     ZERO;               // 0 - reserved for future use
@@ -95,6 +199,8 @@ sub write_update {
     my ( $ns, $selector, $update, $flags ) = @_;
     utf8::encode($ns);
 
+    my $request_id = int( rand( MAX_REQUEST_ID ) );
+
     my $bitflags = 0;
     if ($flags) {
         $bitflags =
@@ -103,11 +209,11 @@ sub write_update {
     }
 
     my $msg =
-        pack( P_UPDATE, 0, int( rand( 2**32 - 1 ) ), 0, OP_UPDATE, 0, $ns, $bitflags )
+        pack( P_UPDATE, 0, $request_id, 0, OP_UPDATE, 0, $ns, $bitflags )
       . $selector
       . $update;
     substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
-    return $msg;
+    return $msg, $request_id;
 }
 
 # struct OP_INSERT {
@@ -123,16 +229,18 @@ sub write_insert {
     my ( $ns, $bson_docs, $flags ) = @_;
     utf8::encode($ns);
 
+    my $request_id = int( rand( MAX_REQUEST_ID ) );
+
     my $bitflags = 0;
     if ($flags) {
         $bitflags = ( $flags->{continue_on_error} ? 1 << I_CONTINUE_ON_ERROR : 0 );
     }
 
     my $msg =
-      pack( P_INSERT, 0, int( rand( 2**32 - 1 ) ), 0, OP_INSERT, $bitflags, $ns )
+      pack( P_INSERT, 0, $request_id, 0, OP_INSERT, $bitflags, $ns )
       . $bson_docs;
     substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
-    return $msg;
+    return $msg, $request_id;
 }
 
 # struct OP_QUERY {
@@ -217,16 +325,18 @@ sub write_delete {
     my ( $ns, $selector, $flags ) = @_;
     utf8::encode($ns);
 
+    my $request_id = int( rand( MAX_REQUEST_ID ) );
+
     my $bitflags = 0;
     if ($flags) {
         $bitflags = ( $flags->{just_one} ? 1 << D_SINGLE_REMOVE : 0 );
     }
 
     my $msg =
-      pack( P_DELETE, 0, int( rand( 2**32 - 1 ) ), 0, OP_DELETE, 0, $ns, $bitflags )
+      pack( P_DELETE, 0, $request_id, 0, OP_DELETE, 0, $ns, $bitflags )
       . $selector;
     substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
-    return $msg;
+    return $msg, $request_id;
 }
 
 # legacy alias
@@ -244,11 +354,14 @@ sub write_delete {
 
 sub write_kill_cursors {
     my (@cursors) = map _pack_cursor_id($_), @_;
+
+    my $request_id = int( rand( MAX_REQUEST_ID ) );
+
     my $msg = pack( P_KILL_CURSORS,
-        0, int( rand( 2**32 - 1 ) ),
+        0, $request_id,
         0, OP_KILL_CURSORS, 0, scalar(@cursors), @cursors );
     substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
-    return $msg;
+    return $msg, $request_id;
 }
 
 # struct {
@@ -281,6 +394,8 @@ sub parse_reply {
     MongoDB::ProtocolError->throw("response was truncated")
         if length($msg) < MIN_REPLY_LENGTH;
 
+    $msg = try_uncompress($msg);
+
     my (
         $len, $msg_id, $response_to, $opcode, $bitflags, $cursor_id, $starting_from,
         $number_returned
