@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,17 +11,17 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
 package MongoDB::_URI;
 
 use version;
-our $VERSION = 'v1.8.3';
+our $VERSION = 'v2.0.0';
 
 use Moo;
 use MongoDB::Error;
+use Encode ();
 use Types::Standard qw(
     Any
     ArrayRef
@@ -33,7 +32,7 @@ use namespace::clean -except => 'meta';
 
 my $uri_re =
     qr{
-            mongodb://
+            mongodb(?:\+srv|)://
             (?: ([^:]*) (?: : ([^@]*) )? @ )? # [username(:password)?@]
             ([^/?]*) # host1[:port1][,host2[:port2],...[,hostN[:portN]]]
             (?:
@@ -98,6 +97,7 @@ sub _build_valid_options {
             authMechanism
             authMechanismProperties
             authSource
+            compressors
             connectTimeoutMS
             connect
             heartbeatFrequencyMS
@@ -108,6 +108,7 @@ sub _build_valid_options {
             readPreference
             readPreferenceTags
             replicaSet
+            retryWrites
             serverSelectionTimeoutMS
             serverSelectionTryOnce
             socketCheckIntervalMS
@@ -116,6 +117,22 @@ sub _build_valid_options {
             w
             wTimeoutMS
             readConcernLevel
+            zlibCompressionLevel
+        )
+    };
+}
+
+has valid_srv_options => (
+    is => 'ro',
+    isa => HashRef,
+    builder => '_build_valid_srv_options',
+);
+
+sub _build_valid_srv_options {
+    return {
+        map { lc($_) => 1 } qw(
+            authSource
+            replicaSet
         )
     };
 }
@@ -123,7 +140,9 @@ sub _build_valid_options {
 sub _unescape_all {
     my $str = shift;
     return '' unless defined $str;
-    $str =~ s/%([0-9a-f]{2})/chr(hex($1))/ieg;
+    if ( $str =~ s/%([0-9a-f]{2})/chr(hex($1))/ieg ) {
+        $str = Encode::decode('UTF-8', $str);
+    }
     return $str;
 }
 
@@ -141,12 +160,176 @@ sub _parse_doc {
     return $set;
 }
 
+sub _parse_options {
+    my ( $self, $valid, $result, $txt_record ) = @_;
+
+    my %parsed;
+    for my $opt ( split '&', $result->{options} ) {
+        my @kv = split '=', $opt, -1;
+        MongoDB::UsageError->throw("expected key value pair") unless @kv == 2;
+        my ( $k, $v ) = map { _unescape_all($_) } @kv;
+        # connection string spec calls for case normalization
+        ( my $lc_k = $k ) =~ tr[A-Z][a-z];
+        if ( !$valid->{$lc_k} ) {
+            if ( $txt_record ) {
+                MongoDB::Error->throw("Unsupported option '$k' in URI $self for TXT record $txt_record\n");
+            } else {
+                warn "Unsupported option '$k' in URI $self\n";
+            }
+            next;
+        }
+        if ( exists $parsed{$lc_k} && !exists $options_with_list_type{$lc_k} ) {
+            warn "Multiple options were found for the same value '$lc_k'. The first occurrence will be used\n";
+            next;
+        }
+        if ( $lc_k eq 'authmechanismproperties' ) {
+            $parsed{$lc_k} = _parse_doc( $k, $v );
+        }
+        elsif ( $lc_k eq 'compressors' ) {
+            $parsed{$lc_k} = [split m{,}, $v, -1];
+        }
+        elsif ( $lc_k eq 'authsource' ) {
+            $result->{db_name} = $v;
+            $parsed{$lc_k} = $v;
+        }
+        elsif ( $lc_k eq 'readpreferencetags' ) {
+            $parsed{$lc_k} ||= [];
+            push @{ $parsed{$lc_k} }, _parse_doc( $k, $v );
+        }
+        elsif ( $lc_k eq 'ssl' || $lc_k eq 'journal' || $lc_k eq 'serverselectiontryonce' ) {
+            $parsed{$lc_k} = __str_to_bool( $k, $v );
+        }
+        else {
+            $parsed{$lc_k} = $v;
+        }
+    }
+    return \%parsed;
+}
+
+sub _fetch_dns_seedlist {
+    my ( $self, $host_name ) = @_;
+
+    my @split_name = split( '\.', $host_name );
+    MongoDB::Error->throw("URI '$self' must contain domain name and hostname")
+        unless scalar( @split_name ) > 2;
+
+    require Net::DNS;
+
+    my $res = Net::DNS::Resolver->new;
+    my $srv_data = $res->query( sprintf( '_mongodb._tcp.%s', $host_name ), 'SRV' );
+
+    my @hosts;
+    my $options = {};
+    my $domain_name = join( '.', @split_name[1..$#split_name] );
+    if ( $srv_data ) {
+        foreach my $rr ( $srv_data->answer ) {
+            next unless $rr->type eq 'SRV';
+            my $target = $rr->target;
+            # search for dot before domain name for a valid hostname - can have sub-subdomain
+            unless ( $target =~ /\.\Q$domain_name\E$/ ) {
+                MongoDB::Error->throw(
+                    "URI '$self' SRV record returns FQDN '$target'"
+                    . " which does not match domain name '${$domain_name}'"
+                );
+            }
+            push @hosts, {
+              target => $target,
+              port   => $rr->port,
+            };
+        }
+        my $txt_data = $res->query( $host_name, 'TXT' );
+        if ( defined $txt_data ) {
+            my @txt_answers;
+            foreach my $rr ( $txt_data->answer ) {
+                next unless $rr->type eq 'TXT';
+                push @txt_answers, $rr;
+            }
+            if ( scalar( @txt_answers ) > 1 ) {
+                MongoDB::Error->throw("URI '$self' returned more than one TXT result");
+            } elsif ( scalar( @txt_answers ) == 1 ) {
+                my $txt_opt_string = join ( '', $txt_answers[0]->txtdata );
+                $options = $self->_parse_options( $self->valid_srv_options, { options => $txt_opt_string }, $txt_opt_string );
+            }
+        }
+    } else {
+        MongoDB::Error->throw("URI '$self' does not return any SRV results");
+    }
+
+    return ( \@hosts, $options );
+}
+
+sub _parse_srv_uri {
+    my ( $self, $uri ) = @_;
+
+    my %result;
+
+    $uri =~ m{^$uri_re$};
+
+    (
+        $result{username}, $result{password}, $result{hostids},
+        $result{db_name},  $result{options}
+    ) = ( $1, $2, $3, $4, $5 );
+
+    $result{hostids} = lc _unescape_all( $result{hostids} );
+
+    if ( !defined $result{hostids} || !length $result{hostids} ) {
+        MongoDB::Error->throw("URI '$self' cannot be empty if using an SRV connection string");
+    }
+
+    if ( $result{hostids} =~ /,/ ) {
+        MongoDB::Error->throw("URI '$self' cannot contain a comma or multiple host names if using an SRV connection string");
+    }
+
+    if ( $result{hostids} =~ /:\d+$/ ) {
+        MongoDB::Error->throw("URI '$self' cannot contain port number if using an SRV connection string");
+    }
+
+    if ( defined $result{options} ) {
+        $result{options} = $self->_parse_options( $self->valid_options, \%result );
+    }
+
+    my ( $hosts, $options ) = $self->_fetch_dns_seedlist( $result{hostids} );
+
+    # Default to SSL on unless specified in conn string options
+    $options = {
+      ssl => 'true',
+      %$options,
+      %{ $result{options} || {} },
+    };
+
+    # URI requires string based booleans for re-constructing the URI
+    if ( ! $options->{ssl} && $options->{ssl} == 0 ) {
+      $options->{ssl} = 'false';
+    }
+
+    my $auth = "";
+    if ( defined $result{username} || defined $result{password} )  {
+        $auth = join(":", map { $_ // "" } $result{username}, $result{password});
+        $auth .= "@";
+    }
+
+    my $new_uri = sprintf(
+        'mongodb://%s%s/%s%s%s',
+        $auth,
+        join( ',', map { sprintf( '%s:%s', $_->{target}, $_->{port} ) } @$hosts ),
+        ($result{db_name} // ""),
+        scalar( keys %$options ) ? '?' : '',
+        join( '&', map { sprintf( '%s=%s', $_, __uri_escape( $options->{$_} ) ) } keys %$options ),
+    );
+
+    return $new_uri;
+}
+
 sub BUILD {
     my ($self) = @_;
 
     my $uri = $self->uri;
     my %result;
 
+    if ( $uri =~ m{^mongodb\+srv://} ) {
+        $uri = $self->_parse_srv_uri( $uri );
+    }
+
     # we throw Error instead of UsageError for errors, to avoid stacktrace revealing credentials
     if ( $uri !~ m{^$uri_re$} ) {
         MongoDB::Error->throw("URI '$self' could not be parsed");
@@ -203,40 +386,7 @@ sub BUILD {
     }
 
     if ( defined $result{options} ) {
-        my $valid = $self->valid_options;
-        my %parsed;
-        for my $opt ( split '&', $result{options} ) {
-            my @kv = split '=', $opt, -1;
-            MongoDB::UsageError->throw("expected key value pair") unless @kv == 2;
-            my ( $k, $v ) = map { _unescape_all($_) } @kv;
-            # connection string spec calls for case normalization
-            ( my $lc_k = $k ) =~ tr[A-Z][a-z];
-            if ( !$valid->{$lc_k} ) {
-                warn "Unsupported option '$k' in URI $self\n";
-                next;
-            }
-            if ( exists $parsed{$lc_k} && !exists $options_with_list_type{$lc_k} ) {
-                warn "Multiple options were found for the same value '$lc_k'\n";
-                next;
-            }
-            if ( $lc_k eq 'authmechanismproperties' ) {
-                $parsed{$lc_k} = _parse_doc( $k, $v );
-            }
-            elsif ( $lc_k eq 'authsource' ) {
-                $result{db_name} = $v;
-            }
-            elsif ( $lc_k eq 'readpreferencetags' ) {
-                $parsed{$lc_k} ||= [];
-                push @{ $parsed{$lc_k} }, _parse_doc( $k, $v );
-            }
-            elsif ( $lc_k eq 'ssl' || $lc_k eq 'journal' || $lc_k eq 'serverselectiontryonce' ) {
-                $parsed{$lc_k} = __str_to_bool( $k, $v );
-            }
-            else {
-                $parsed{$lc_k} = $v;
-            }
-        }
-        $result{options} = \%parsed;
+        $result{options} = $self->_parse_options( $self->valid_options, \%result );
     }
 
     for my $attr (qw/username password db_name options hostids/) {
@@ -275,8 +425,8 @@ sub __uri_escape {
     return $str;
 }
 
-# Check if should have been escaped; allow safe chars plus '+' and '%'
-my $unreserved = q[a-z0-9._~-]; # use last so it ends in '-'
+# Rules for valid userinfo from RFC 3986 Section 3.2.1.
+my $unreserved = q[a-z0-9._~-]; # use this class last so regex ends in '-'
 my $subdelimit = q[!$&'()*+,;=];
 my $allowed = "%$subdelimit$unreserved";
 my $not_allowed_re = qr/[^$allowed]/i;
